/* JAI-Ext - OpenSource Java Advanced Image Extensions Library
 *    http://www.geo-solutions.it/
 *    Copyright 2018 GeoSolutions
 *    
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


/*
 * Copyright (c) 2018, Michael Bedward. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer.
 *
 * - Redistributions in binary form must reproduce the above copyright notice, this
 *   list of conditions and the following disclaimer in the documentation and/or
 *   other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package it.geosolutions.jaiext.jiffle.parser.node;

import java.util.Objects;

public class LoopInRange implements Statement {

    private Statement statement;
    private Variable loopVariable;
    private Expression low;
    private Expression high;

    public LoopInRange(Variable loopVariable, Expression low, Expression high, Statement statement) {
        this.loopVariable = loopVariable;
        this.low = low;
        this.high = high;
        this.statement = statement;
    }

    @Override
    public void write(SourceWriter w) {
        String lowVariable = "_lo" + loopVariable;
        String highVariable = "_hi" + lowVariable;
        w.indent().append("final int ").append(lowVariable).append(" = (int) (").append(low).append(");").newLine();
        w.indent().append("final int ").append(highVariable).append(" = (int) (").append(high).append(");").newLine();
        w.indent().append("for(int ").append(loopVariable).append(" = ").append(lowVariable);
        w.append("; ").append(loopVariable).append(" <= ").append(highVariable);
        w.append("; ").append(loopVariable).append("++) {").newLine();
        w.inc();
        statement.write(w);
        w.dec();
        w.line("}");
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        LoopInRange that = (LoopInRange) o;
        return Objects.equals(statement, that.statement) &&
                Objects.equals(loopVariable, that.loopVariable) &&
                Objects.equals(low, that.low) &&
                Objects.equals(high, that.high);
    }

    @Override
    public int hashCode() {
        return Objects.hash(statement, loopVariable, low, high);
    }

    public Statement getStatement() {
        return statement;
    }

    public Variable getLoopVariable() {
        return loopVariable;
    }

    public Expression getLow() {
        return low;
    }

    public Expression getHigh() {
        return high;
    }
}
